"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020 LabWare
Copyright (C) 2022 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:vdb' }"

"{ NameSpace: Smalltalk }"

VDBAbstractContentsApplication subclass:#VDBAbstractContainer
	instanceVariableNames:'components labels'
	classVariableNames:''
	poolDictionaries:''
	category:'VDB-UI-Abstract'
!

!VDBAbstractContainer class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020 LabWare
Copyright (C) 2022 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!VDBAbstractContainer class methodsFor:'queries'!

isAbstract
    "Return if this class is an abstract class.
     True is returned here for myself only; false for subclasses.
     Abstract subclasses must redefine again."

    ^ self == VDBAbstractContainer.
! !

!VDBAbstractContainer methodsFor:'adding & removing components'!

addApplication: anApplicationModel
    | label |

    label := anApplicationModel perform: #title ifNotUnderstood:[ anApplicationModel class name ].
    ^ self addApplication: anApplicationModel labeled: label

    "Created: / 10-06-2014 / 14:23:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 10-06-2014 / 16:59:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

addApplication: anApplicationModel labeled: aString
    ^ self addApplication: anApplicationModel labeled: aString beforeIndex: components size + 1

    "Created: / 10-06-2014 / 16:59:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

addApplication: anApplicationModel labeled: aString beforeIndex: anInteger
    | component |

    component := ApplicationSubView new.
    anApplicationModel createBuilder.
    anApplicationModel window: component.
    component client: anApplicationModel.
    ^ self addComponent: component labeled: aString beforeIndex: anInteger

    "Created: / 10-06-2014 / 16:58:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

addComponent: aView labeled: aString 
    self addComponent: aView labeled: aString beforeIndex: labels size + 1.

    "Created: / 10-06-2014 / 16:39:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

addComponent: aView labeled: aString beforeIndex: anInteger
    labels add: aString beforeIndex: anInteger.
    components add: aView beforeIndex: anInteger.
    self assert: labels size == components size.

    "Created: / 10-06-2014 / 16:37:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 17-09-2014 / 22:44:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBAbstractContainer methodsFor:'aspects'!

debuggerHolder:aValueModel
    super debuggerHolder:aValueModel.
    components do:[:each | 
        each isApplicationSubView ifTrue:[ 
            each application debuggerHolder: aValueModel
        ].
    ].

    "Created: / 10-06-2014 / 17:27:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!VDBAbstractContainer methodsFor:'change & update-delayed'!

delayedUpdateContents
    "Nothing by default"

    "Created: / 18-06-2020 / 23:32:30 / Jan Vrany <jan.vrany@labware.com>"
! !

!VDBAbstractContainer methodsFor:'initialization & release'!

initialize
    super initialize.
    labels := List new.
    components := List new.
    components addDependent: self.

    "Created: / 10-06-2014 / 14:24:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 17-09-2014 / 22:21:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 08-08-2020 / 22:35:38 / Jan Vrany <jan.vrany@labware.com>"
! !

!VDBAbstractContainer class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

