"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020-2022 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:vdb' }"

"{ NameSpace: Smalltalk }"

LibraryDefinition subclass:#jv_vdb
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* Projects & Packages *'
!

!jv_vdb class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020-2022 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
! !

!jv_vdb class methodsFor:'description'!

excludedFromPreRequisites
    "list packages which are to be explicitely excluded from the automatic constructed
     prerequisites list. If empty, everything that is found along the inheritance of any of
     my classes is considered to be a prerequisite package."

    ^ #(
    )
!

mandatoryPreRequisites
    "list packages which are mandatory as a prerequisite.
     This are packages containing superclasses of my classes and classes which
     are extended by myself.
     They are mandatory, because we need these packages as a prerequisite for loading and compiling.
     This method is generated automatically,
     by searching along the inheritance chain of all of my classes.
     Please take a look at the #referencedPreRequisites method as well."

    ^ #(
        #'jv:libgdbs'    "GDBCommand - superclass of VDBMI_vdb_var_create"
        #'stx:goodies/announcements'    "Announcement - superclass of RREvent"
        #'stx:goodies/petitparser'    "PPCompositeParser - superclass of VDBParser"
        #'stx:libbasic'    "LibraryDefinition - superclass of jv_vdb"
        #'stx:libtool'    "AbstractSettingsApplication - superclass of VDBSettingsApplication"
        #'stx:libview'    "DisplaySurface - superclass of VDBSimpleConsoleView"
        #'stx:libview2'    "ApplicationModel - superclass of VDBAbstractApplication"
        #'stx:libwidg'    "EditTextView - superclass of VDBSimpleConsoleView"
        #'stx:libwidg2'    "AbstractHierarchicalItem - superclass of VDBAbstractPresenter"
    )
!

referencedPreRequisites
    "list packages which are a prerequisite, because they contain
     classes which are referenced by my classes.
     These packages are NOT needed as a prerequisite for compiling or loading,
     however, a class from it may be referenced during execution and having it
     unloaded then may lead to a runtime doesNotUnderstand error, unless the caller
     includes explicit checks for the package being present.
     This method is generated automatically,
     by searching all classes (and their packages) which are referenced by my classes.
     Please also take a look at the #mandatoryPreRequisites method"

    ^ #(
        #'stx:goodies/xml/vw'    "XML::XMLParser - referenced by VDBVirtualMemoryMap>>initializeFromSysInternalsVmmapDump:"
        #'stx:libbasic2'    "List - referenced by VDBAbstractContainer>>initialize"
        #'stx:libcomp'    "SyntaxHighlighter - referenced by VDBScatchPadApplication>>loadFile:encoding:into:"
        #'stx:libtool2'    "MenuEditor - referenced by VDBIconLibrary class>>initialize"
    )
!

subProjects
    "list packages which are known as subprojects. 
     The generated makefile will enter those and make there as well.
     However: they are not forced to be loaded when a package is loaded; 
     for those, redefine requiredPrerequisites."

    ^ #(
    )
! !

!jv_vdb class methodsFor:'description - compilation'!

additionalRules_bc_dot_mak
    ^ '
python:            

$(INSTALLLIB_DIR)\python:
        md $@

$(INSTALLLIB_DIR)\python\vdb.py: python\vdb.py $(INSTALLLIB_DIR)\python
        copy python\vdb.py $@

install:: $(INSTALLLIB_DIR)\python\vdb.py

'

    "Created: / 11-07-2019 / 22:27:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 12-07-2019 / 10:26:06 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 17-05-2021 / 11:46:13 / Jan Vrany <jan.vrany@labware.com>"
!

additionalRules_make_dot_proto
    ^ '
python:

$(INSTALLLIB_DIR)/python:
        mkdir -p $@

$(INSTALLLIB_DIR)/python/vdb.py: python/vdb.py $(INSTALLLIB_DIR)/python
        $(INSTALL) $< $@

install:: $(INSTALLLIB_DIR)/python/vdb.py
'

    "Created: / 11-07-2019 / 22:30:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-07-2019 / 08:13:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 17-05-2021 / 11:46:24 / Jan Vrany <jan.vrany@labware.com>"
!

additionalTargets_bc_dot_mak
    "obsolete - kept for compatibility with old project files"

    ^ 'python'

    "Created: / 17-05-2021 / 11:51:17 / Jan Vrany <jan.vrany@labware.com>"
!

additionalTargets_make_dot_proto
    "allows for additional targets to be added to the make.proto file."

    ^ 'python'

    "Created: / 17-05-2021 / 11:51:10 / Jan Vrany <jan.vrany@labware.com>"
! !

!jv_vdb class methodsFor:'description - contents'!

classNamesAndAttributes
    "lists the classes which are to be included in the project.
     Each entry in the list may be: a single class-name (symbol),
     or an array-literal consisting of class name and attributes.
     Attributes are: #autoload or #<os> where os is one of win32, unix,..."

    ^ #(
        "<className> or (<className> attributes...) in load order"
        RR
        RREvent
        VDBAbstractApplication
        VDBAbstractPresenter
        VDBAddress
        VDBAddressRange
        VDBEvaluator
        VDBIconLibrary
        VDBInstructionBasicBlock
        #'VDBMI_vdb_var_create'
        #'VDBMI_vdb_var_delete'
        #'VDBMI_vdb_var_evaluate_expression'
        #'VDBMI_vdb_var_info_path_expression'
        #'VDBMI_vdb_var_list_children'
        #'VDBMI_vdb_var_set_visualizer'
        #'VDBMI_vdb_var_update'
        VDBParser
        VDBScatchPadApplication
        VDBSettingsApplication
        VDBSimpleConsoleView
        VDBStartup
        VDBTerminalView
        VDBVariableObject
        VDBVariableObjectExecutor
        VDBVirtualMemoryMap
        VDBVirtualMemoryRegion
        #'jv_vdb'
        RRExitEvent
        RRStartEvent
        VDBAbstractConsoleApplication
        VDBAbstractContentsApplication
        VDBBreakpointApplication
        VDBBreakpointPresenter
        VDBDebuggerApplication
        VDBEventLogApplication
        VDBFramePresenter
        VDBInstructionPresenter
        VDBInstructionsAndSourcePresenter
        #'VDBMI_vdb_var_duplicate'
        VDBMemoryApplication
        VDBRegisterPresenter
        VDBSourceApplication
        VDBStatusApplication
        VDBThreadGroupPresenter
        VDBThreadPresenter
        VDBVariableObjectPresenter
        VDBAbstractContainer
        VDBAbstractListApplication
        VDBAbstractUnixConsoleApplication
        VDBDisassemblyApplication
        VDBSimpleDebuggerConsoleApplication
        VDBAbstractTreeApplication
        VDBTabbingContainer
        VDBUnixDebuggerConsoleApplication
        VDBUnixInferiorConsoleApplication
        VDBUnixReplayServerConsoleApplication
        VDBBreakpointListApplication
        VDBFrameApplication
        VDBInstructionListApplication
        VDBRegisterListApplication
        VDBSourceAndDisassemblyApplication
        VDBStackApplication
        VDBVariableObjectListApplication
    )
!

extensionMethodNames
    "lists the extension methods which are to be included in the project.
     Entries are 2-element array literals, consisting of class-name and selector.
     A correponding method with real names must be present in my concrete subclasses
     if it has extensions."

    ^ #(
        GDBVariableObject inspector2TabVariable
        GDBDebugger targetConnectRR:
        GDBDebugger onRRExitEvent:
        GDBDebugger pythonImportVDB
        GDBVariableObject varobj
        GDBFrame dumpOn:
        GDBFrame dumpString
        GDBVariable dumpOn:
    )
! !

!jv_vdb class methodsFor:'description - project information'!

companyName
    "Returns a company string which will appear in <lib>.rc.
     Under win32, this is placed into the dlls file-info"

    ^ 'Jan Vrany'

    "Modified: / 23-11-2017 / 22:33:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

description
    "Returns a description string which will appear in nt.def / bc.def"

    ^ 'Visual / VM Debugger Library'

    "Modified: / 23-11-2017 / 22:34:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

legalCopyright
    "Returns a copyright string which will appear in <lib>.rc.
     Under win32, this is placed into the dlls file-info"

    ^ 'Copyright (C) Jan Vrany 2015-now'

    "Modified (format): / 23-11-2017 / 22:36:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

productName
    "Returns a product name which will appear in <lib>.rc.
     Under win32, this is placed into the dlls file-info.
     This method is usually redefined in a concrete application definition"

    ^ self description

    "Modified: / 23-11-2017 / 22:36:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!jv_vdb class methodsFor:'file generation'!

protectedFileNames
    "names of files which should NOT be generated (because they are hand-maintained)
     - redefine this to protect a hand-written Make.proto or other handwritten support files"

    ^ #('vdbinit.st')

    "Created: / 17-11-2020 / 15:23:46 / Jan Vrany <jan.vrany@labware.com>"
! !

!jv_vdb class methodsFor:'documentation'!

version_HG
    ^ '$Changeset: <not expanded> $'
! !

